/* Copyright (C) 2000-2015 Lavtech.com corp. All rights reserved.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
*/

#ifndef _UDM_COMMON_H
#define _UDM_COMMON_H

#include "udm_config.h"

#include <stddef.h>

#include <stdio.h> /* for FILE etc. */

#include <sys/types.h>

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#ifdef HAVE_WINSOCK_H
#include <winsock.h>
#endif
#ifdef HAVE_SYS_SOCKET_H
#include <sys/socket.h>
#endif
#ifdef HAVE_NETINET_IN_H
#include <netinet/in.h>
#endif
#ifdef HAVE_ARPA_INET_H
#include <arpa/inet.h>
#endif
#ifdef HAVE_ARPA_NAMESER_H
#include <arpa/nameser.h>
#endif
#ifdef HAVE_RESOLV_H
#include <resolv.h>
#endif
#ifdef HAVE_NETDB_H
#include <netdb.h>
#endif
#ifdef MECAB
#include <mecab.h>
#endif

#ifdef HAVE_DEBUG
#include <assert.h>
#endif

/************************ Basic data types ****************/

#if 0
#if 0
typedef const char * udm_rc_t;
#define UDM_OK			((const char*)NULL)
#define UDM_ERROR		((const char*)"error")
#define UDM_NOTARGET		((const char*)"notarget")
#define UDM_TERMINATED		((const char*)"terminated")
#else
typedef unsigned long udm_rc_t;
#define UDM_OK         0
#define UDM_ERROR      0xFFFFFFFFUL
#define UDM_NOTARGET   0xFFFFFFFEUL
#define UDM_TERMINATED 0xFFFFFFFDUL
#endif
#else

typedef enum
{
  UDM_OK= 0,
  UDM_ERROR= 1,
  UDM_NOTARGET= 2,
  UDM_TERMINATED= 3
} udm_rc_t;
#endif

typedef enum 
{
  UDM_FALSE= 0,
  UDM_TRUE= 1
} udm_bool_t;


#define UDM_TEST(x) ((x) ? UDM_TRUE : UDM_FALSE)

#ifndef udm_max
#define udm_max(a,b) (((a) > (b)) ? (a) : (b))
#endif
#ifndef udm_min
#define udm_min(a,b) (((a) < (b)) ? (a) : (b))
#endif

typedef int (*udm_qsort_cmp)(const void*, const void*);

typedef uint4 udm_pos_t;
typedef unsigned char udm_secno_t;
typedef unsigned char udm_wordnum_t;

/**********************************************************/

#include "udm_unicode.h"
#include "udm_uniconv.h"
#include "udm_unidata.h"
#include "udm_hash.h"

/**************************** Constant strings *******************************/
typedef struct udm_const_string_st
{
  const char *str;
  size_t length;
} UDM_CONST_STR;

/************************* Constant attribute - name with value *************/
typedef struct udm_const_attr_st
{
  UDM_CONST_STR name;
  UDM_CONST_STR value;
} UDM_CONST_ATTR;

/**************************** Variable strings *******************************/
typedef struct udm_str_st
{
  char *str;
  size_t length;
} UDM_STR;

/************************ Dynamic strings ************************************/
typedef struct dstr_struct
{
  size_t size_alloced; /* Bytes allocated */
  size_t size_page;    /* Bytes to allocate on overflow */
#ifdef DSTR_PRIVATE
  UDM_STR Val;         /* Value (usually null-terminated string) and its length */
#else
  UDM_STR Val;
#endif
} UDM_DSTR;

#include "udm_utils.h"


/* Some constants */
#define UDM_LANGPERDOC				16		/* FIXME */
#define UDM_USER_AGENT				"MnoGoSearch/" VERSION
#define UDM_MAXWORDPERQUERY			64

/* Some sizes and others definitions */
#define UDM_MAXDOCSIZE				2*1024*1024	/**< 2 MB  */
#define UDM_DEFAULT_REINDEX_TIME		7*24*60*60	/**< 1week */
#define UDM_MAXWORDSIZE				32
#define UDM_MAXDISCWORDSIZE			64
#define UDM_DEFAULT_MAX_HOPS			256
#define UDM_READ_TIMEOUT			30
#define UDM_DOC_TIMEOUT				90
#define UDM_MAXNETERRORS			16
#define UDM_DEFAULT_NET_ERROR_DELAY_TIME	86400
#define UDM_DEFAULT_BAD_SINCE_TIME              15*24*60*60     /**< 15 days */
#define UDM_FINDURL_CACHE_SIZE                  128
#define UDM_SERVERID_CACHE_SIZE                 128
#define UDM_ERRSTR_SIZE                         2048

/* search modes */
typedef enum
{
  UDM_MODE_ALL=             0,
  UDM_MODE_ANY=             1,
  UDM_MODE_BOOL=            2,
  UDM_MODE_PHRASE=          3,
  UDM_MODE_ALL_MINUS=       4,
  UDM_MODE_ALL_MINUS_HALF=  5
} udm_search_mode_t;


/* word and pattern match type */
typedef enum
{
  UDM_MATCH_FULL=       0,
  UDM_MATCH_BEGIN=      1,
  UDM_MATCH_SUBSTR=     2,
  UDM_MATCH_END=        3,
  UDM_MATCH_REGEX=      4,
  UDM_MATCH_WILD=       5,
  UDM_MATCH_SUBNET=     6,
  UDM_MATCH_NUMERIC_LT= 7,
  UDM_MATCH_NUMERIC_GT= 8,
  UDM_MATCH_RANGE=      9
} udm_match_mode_t;


/* Case sensitivity */
#define UDM_CASE_SENSITIVE      0
#define UDM_CASE_INSENSITIVE    1

/* Flags for indexing */
#define UDM_FLAG_REINDEX           1
#define UDM_FLAG_SORT_EXPIRED      2
#define UDM_FLAG_SORT_HOPS         4
#define UDM_FLAG_ADD_SERV          8
#define UDM_FLAG_SPELL	          16
#define UDM_FLAG_LOAD_LANGMAP	    32
#define UDM_FLAG_DONTSORT_SEED    64
#define UDM_FLAG_ADD_SERVURL	   128
#define UDM_FLAG_DONT_ADD_TO_DB  256


/* URLFile actions */
typedef enum
{
  UDM_URL_FILE_REINDEX= 1,
  UDM_URL_FILE_CLEAR=   2,
  UDM_URL_FILE_INSERT=  3,
  UDM_URL_FILE_PARSE=   4
} udm_urlfilecmd_t;

/* Ispell mode binary flags */
#define UDM_ISPELL_MODE_DB	1
#define UDM_ISPELL_USE_PREFIXES	2
#define UDM_ISPELL_MODE_SERVER  4


/* Known content types */
typedef enum 
{
  UDM_CONTENT_TYPE_UNKNOWN= 0,
  UDM_CONTENT_TYPE_TEXT_PLAIN= 1,
  UDM_CONTENT_TYPE_TEXT_HTML= 2,
  UDM_CONTENT_TYPE_TEXT_XML= 3,
  UDM_CONTENT_TYPE_MESSAGE_RFC822= 4,
  UDM_CONTENT_TYPE_AUDIO_MPEG= 5,
  UDM_CONTENT_TYPE_HTDB= 6,
  UDM_CONTENT_TYPE_DOCX= 7,
  UDM_CONTENT_TYPE_TEXT_RTF= 8
} udm_content_type_t;


/* Action type: HTTP methods */
typedef enum
{
  UDM_METHOD_GET=          0,
  UDM_METHOD_UNKNOWN=      1,
  UDM_METHOD_DISALLOW=     2,
  UDM_METHOD_HEAD=         3,
  UDM_METHOD_HREFONLY=     4,
  UDM_METHOD_CHECKMP3=     5,
  UDM_METHOD_CHECKMP3ONLY= 6,
  UDM_METHOD_VISITLATER=   7,
  UDM_METHOD_INDEX=        8,
  UDM_METHOD_NOINDEX=      9,
  UDM_METHOD_IMPORTONLY=   10
} udm_method_t;

#define UDM_METHOD_DEFAULT      UDM_METHOD_GET

typedef enum
{
  UDM_LINK_SOURCE_UNKNOWN=         0,
  UDM_LINK_SOURCE_CMDLINE=         1,
  UDM_LINK_SOURCE_CONF=            2,
  UDM_LINK_SOURCE_A_HREF=          3,
  UDM_LINK_SOURCE_FRAME_SRC=       4,
  UDM_LINK_SOURCE_IMG_SRC=         5,
  UDM_LINK_SOURCE_REDIRECT=        6,
  UDM_LINK_SOURCE_META_REFRESH=    7,
  UDM_LINK_SOURCE_LINK_HREF=       8,
  UDM_LINK_SOURCE_AREA_HREF=       9,
  UDM_LINK_SOURCE_IFRAME_SRC=     10,
  UDM_LINK_SOURCE_SCRIPT_SRC=     11,
  UDM_LINK_SOURCE_HTDB=           12,
  UDM_LINK_SOURCE_URLFILE=        13,
  UDM_LINK_SOURCE_ROBOTS_SITEMAP= 14,
  UDM_LINK_SOURCE_XML=            15,
  UDM_LINK_SOURCE_DIR=            16  /* Directory listing: file:///dir/ */
} udm_link_source_t;
#define UDM_LINK_SOURCES_YES 0x7FFFFFFF


/* CollectLinks flags*/
typedef enum
{
  UDM_COLLECT_LINKS_NONE=      0x0000,  /* do not collect links           */
  UDM_COLLECT_LINKS_INNER=     0x0001,  /* links covered by .conf file    */
  UDM_COLLECT_LINKS_OUTER=     0x0002,  /* links going outside .conf file */
  UDM_COLLECT_LINKS_SITE=      0x0004,  /* links going to the same site   */
  UDM_COLLECT_LINKS_PAGE=      0x0008,  /* links going to the same page   */
  UDM_COLLECT_LINKS_BADSCHEME= 0x0010,  /* Schema: mailto, javascript     */
  UDM_COLLECT_LINKS_BAD=       0x0020,  /* Bad syntax                     */
  UDM_COLLECT_LINKS_HOPS=      0x0040,  /* Too many hops                  */
  UDM_COLLECT_LINKS_FILTER=    0x0080,  /* Too many hops                  */
  UDM_COLLECT_LINKS_PERSITE=   0x0100,  /* Per site limite reached        */
  UDM_COLLECT_LINKS_ALL_DST=   0xFFFF
} udm_link_destination_t;


/*
  Don't need these by default:
  UDM_COLLECT_LINKS_PAGE
  UDM_COLLECT_LINKS_BADSCHEMA
  UDM_COLLECT_LINKS_BAD
*/
#define UDM_COLLECT_LINKS_YES \
  (UDM_COLLECT_LINKS_INNER   |\
   UDM_COLLECT_LINKS_OUTER   |\
   UDM_COLLECT_LINKS_SITE    |\
   UDM_COLLECT_LINKS_HOPS    |\
   UDM_COLLECT_LINKS_FILTER  |\
   UDM_COLLECT_LINKS_PERSITE)
#define UDM_COLLECT_LINKS_DEFAULT UDM_COLLECT_LINKS_YES

/* Robots flags */
typedef enum
{
  UDM_ROBOTS_NONE=   0x0000, /* Don't respect robots limitation  */
  UDM_ROBOTS_TXT=    0x0001, /* Respect robots.txt instructions  */
  UDM_ROBOTS_HEADER= 0x0002, /* Restext "X-Robots-Tag" HTTP header        */
  UDM_ROBOTS_META=   0x0004, /* Respect <meta robots="xxx">  insturctions */
  UDM_ROBOTS_REL=    0x0008, /* Respect <link rel="nofollow> insturctions */
  UDM_ROBOTS_ALL=    0xFFFF  /* Respect all robots instructions  */
} udm_use_robots_t;


typedef enum
{
  UDM_COLLECT_LINKS_ABSOLUTE=  0x000000, /* Store in absolute format   */
  UDM_COLLECT_LINKS_ASIS=      0x010000, /* Preserve the original form */
  UDM_COLLECT_LINKS_FORMAT=    0xFF0000
} udm_link_format_t;


/* Content encoding types */
typedef enum
{
  UDM_CONTENT_ENCODING_IDENTITY= 0,
  UDM_CONTENT_ENCODING_DEFLATE=  1,
  UDM_CONTENT_ENCODING_GZIP=     3,
  UDM_CONTENT_ENCODING_ZLIB_COMPRESS= 4,
  UDM_CONTENT_ENCODING_UNKNOWN=  5
} udm_content_encoding_t;


/* Words origins */
typedef enum
{
  UDM_WORD_ORIGIN_QUERY=           1,
  UDM_WORD_ORIGIN_SPELL=           2,
  UDM_WORD_ORIGIN_SYNONYM=         3,
  UDM_WORD_ORIGIN_SYNONYM_FINAL=   4,
  UDM_WORD_ORIGIN_STOP=            5,
  UDM_WORD_ORIGIN_SUGGEST=         6,
  UDM_WORD_ORIGIN_COLLATION=       7
} udm_wordorigin_t;


/* URL data flags */
#define UDM_URLDATA_URL	        1
#define UDM_URLDATA_SITE        2
#define UDM_URLDATA_POP         4
#define UDM_URLDATA_LM          8 
#define UDM_URLDATA_SU         16
#define UDM_URLDATA_SITE_RANK  32

/* Locking mutex numbers */
#define UDM_LOCK_INTERNAL       0
#define UDM_LOCK_CONF		1
#define UDM_LOCK_WIN		2
#define UDM_LOCK_THREAD         3
#define UDM_LOCK_SQL            4
#define UDM_LOCK_SEGMENTER      5
#define UDM_LOCK_DB             6
#define UDM_LOCK_LOG		7
#define UDM_LOCK_HREF_CACHE	8
#define UDM_LOCK_TARGETS        9
#define UDM_LOCK_HOST_CACHE     10
#define UDM_LOCK_INADDR_CACHE   11
#define UDM_LOCK_DOC_CACHE      12
#define UDM_LOCK_ROBOT_CACHE    13
#define UDM_LOCK_COOKIE_CACHE   14
/* Don't forget to add a new name into mutex.c */
#define UDM_LOCK_ROBOT_FIRST	15
#define UDM_LOCK_ROBOT_LAST	128
#define UDM_LOCK_ROBOT_COUNT    (UDM_LOCK_ROBOT_LAST - UDM_LOCK_ROBOT_FIRST +1)
#define UDM_LOCK_HOST_FIRST     129
#define UDM_LOCK_HOST_LAST      255
#define UDM_LOCK_HOST_COUNT     (UDM_LOCK_HOST_LAST - UDM_LOCK_HOST_FIRST + 1)
#define UDM_LOCK_MAX            (UDM_LOCK_HOST_LAST + 1)

typedef unsigned int udm_threadid_t;
typedef unsigned int udm_mutexno_t;

typedef enum
{
  UDM_LOCK,
  UDM_UNLOCK,
  UDM_CKLOCK
} udm_mutexcmd_t;

/************************ Statistics **********************/
typedef struct stat_struct
{
  int status;
  int expired;
  int total;
} UDM_STAT;

typedef struct stat_list_struct
{
  time_t   time;
  size_t   nstats;
  UDM_STAT *Stat;
} UDM_STATLIST;

/************************ VARLISTs ************************/

/* Various variable flags */
typedef enum
{
  UDM_VARFLAG_NONE=       0x00,
  UDM_VARFLAG_NOCLONE=    0x01,  /* Ignore for clone detection        */ 
  UDM_VARFLAG_USERDEF=    0x02,  /* User defined section              */
  UDM_VARFLAG_HTMLSOURCE= 0x08,  /* If apply HTML parser, for HTDB    */
  UDM_VARFLAG_WIKI=       0x10,  /* If to remove text between [ and ] */
  UDM_VARFLAG_HL=         0x20,  /* If variable has highlight markers */
  UDM_VARFLAG_NOINDEX=    0x40,  /* If section should be in bdicti but not in bdict */
  UDM_VARFLAG_DECIMAL=    0x80,  /* Whether to detect decimal numbers */
  UDM_VARFLAG_RAW=       0x100,  /* If sections is Raw (no cs conversion */
  UDM_VARFLAG_READONLY=  0x200,  /* If read only (e.g. full) */
  UDM_VARFLAG_FORCETXT=  0x400   /* $(var) forces TXT rather than HTML output */
} udm_var_flag_t;


/* Value handler types */
typedef enum
{
  UDM_VALUE_HANDLER_TYPE_STR=               1,
  UDM_VALUE_HANDLER_TYPE_INT=               2,
  UDM_VALUE_HANDLER_TYPE_DOUBLE=            3,
  UDM_VALUE_HANDLER_TYPE_CHAR=              4,
  UDM_VALUE_HANDLER_TYPE_ENV=              16,
  UDM_VALUE_HANDLER_TYPE_RESULT=           18,
  UDM_VALUE_HANDLER_TYPE_DOCUMENT=         19,
  UDM_VALUE_HANDLER_TYPE_SQLRESULT=        20,
  UDM_VALUE_HANDLER_TYPE_EXCERPT_FRAGMENT= 21
} udm_value_handler_type_t;

/*
  Value handler data types.
  Handlers of different type can have the data type.
*/
typedef enum
{
  UDM_VALUE_DATA_TYPE_STR=               1,
  UDM_VALUE_DATA_TYPE_INT=               2,
  UDM_VALUE_DATA_TYPE_DOUBLE=            3,
  UDM_VALUE_DATA_TYPE_CHAR=              4,
  UDM_VALUE_DATA_TYPE_ENV=              16,
  UDM_VALUE_DATA_TYPE_RESULT=           18,
  UDM_VALUE_DATA_TYPE_DOCUMENT=         19,
  UDM_VALUE_DATA_TYPE_SQLRESULT=        20,
  UDM_VALUE_DATA_TYPE_EXCERPT_FRAGMENT= 21
} udm_value_data_type_t;



/*
  These values should not be equal to any secno values
  in message_header_param[] in message.c.
  TODO34: get rid of this. Don't put QSTRING and ENV
  values into ENV, e.g. use UDM_QUERY instead.
*/
#define UDM_VARSRC_QSTRING 255
#define UDM_VARSRC_ENV     254


typedef struct
{
  udm_secno_t        secno;    /**< Number 0..255   */
  udm_var_flag_t     flags;  
} UDM_SECTION_PARAM;


typedef struct
{
  UDM_DSTR          Value;    /**< Value with length  */
  UDM_SECTION_PARAM Param;  /**< Parameters         */
} UDM_SECTION;


struct udm_value_st;
struct udm_var_st;
struct udm_varlist_st;
struct udm_value_handler_st;
struct udm_prog_executor_state_st;
struct udm_prog_executor_st;

typedef struct udm_value_prototype_st
{
  const struct udm_value_handler_st *handler;
} UDM_VALUE_PROTOTYPE;


typedef struct udm_function_prototype_st
{
  UDM_VALUE_PROTOTYPE result;
  size_t nargs;
  const UDM_VALUE_PROTOTYPE *args;
} UDM_FUNCTION_PROTOTYPE;


typedef void (*udm_func_runtime_t)(struct udm_prog_executor_state_st *state);

typedef struct
{
  const UDM_CONST_STR name;
  UDM_FUNCTION_PROTOTYPE prototype;
  udm_func_runtime_t func;
} UDM_FUNCTION;


typedef struct udm_value_handler_st
{
  udm_value_handler_type_t type;
  udm_value_data_type_t datatype;
  udm_value_data_type_t native_reg_type;
  const char *type_name;
  udm_rc_t (*Constructor)(char *Data,
                          const struct udm_value_st **args, size_t nargs);
  void (*Destructor)(char *Data);

  udm_rc_t (*Copy)(char *To, const char *From);

  size_t (*VarSize)(void);
  size_t (*ValueSize)(void);
  size_t (*DataOffset)(void);
  size_t (*DataSize)(void);
  size_t (*DataAlignment)(void);
  udm_rc_t (*Dump)(const char *Data, const char *name, FILE *f);
  udm_var_flag_t (*Flags)(const char *Data);
  udm_secno_t (*Secno)(const char *Data);
  size_t (*MemUsed)(const char *Data);

  void (*GetConstStr)(const char *D, UDM_CONST_STR *Val);
  void (*GetBool)(const char *D, udm_bool_t *val);
  void (*GetInt)(const char *D, int *val);
  void (*GetUInt)(const char *D, unsigned int *val);
  void (*GetDouble)(const char *D, double *val);
  udm_rc_t (*SetInt)(char *D, int value);
  udm_rc_t (*SetDouble)(char *D, double value);
  udm_rc_t (*SetStrn)(char *D, const char *str, size_t length);

  udm_rc_t (*PrintToFile)(const char *D, FILE *f);
  udm_rc_t (*PrintToDSTR)(const char *D, UDM_DSTR *dstr);

  udm_rc_t (*ConvertCharset)(char *Data, UDM_CONV *conv, int flags);
  /* Set new value but don't touch the other members (e.g. section param) */
  udm_rc_t (*SetConv)(char *Data, UDM_CONV *cnv, int cnvflag, const char *src, size_t length);
  udm_rc_t (*AppendStrn)(char *Data, const char *str, size_t length);
  udm_rc_t (*AppendConv)(char *Data, size_t maxlen, UDM_CONV *cnv, int cnvflags, const char *src, size_t length);

  const UDM_FUNCTION *method;
} UDM_VALUE_HANDLER;


typedef struct udm_value_st
{
  const struct udm_value_handler_st *handler; /**< Value handler  */
} UDM_VALUE;


typedef struct udm_var_header_st
{
  char                      *name;  /**< Variable name */
} UDM_VAR_HEADER;

typedef struct udm_var_st
{
  UDM_VAR_HEADER            header;
#ifdef UDM_VAR_H_PRIVATE
  UDM_VALUE                 Value;
#else
  UDM_VALUE                 m_hidden_Value;
#endif
} UDM_VAR;


typedef enum
{
  UDM_VARLIST_FLAG_NONE= 0,
  UDM_VARLIST_FLAG_CS= 1  /* Case sensitive */
} udm_varlist_flag_t;


typedef struct udm_varlist_st
{
  size_t   nvars; /* Number of registered variables */
  size_t   mvars; /* Number of allocated variables  */
  UDM_VAR  **Var;
  udm_varlist_flag_t flags;
} UDM_VARLIST;


typedef struct
{
  size_t nitems;
  size_t mitems;
  UDM_VARLIST *Item;
} UDM_VARLISTLIST;


typedef enum
{
  UDM_TEXTLIST_FLAG_NONE=             0x00,
  UDM_TEXTLIST_FLAG_SKIP_ADD_SECTION= 0x01,
  UDM_TEXTLIST_FLAG_RFC1522=          0x02, /* Message header (Subj, From)*/
  UDM_TEXTLIST_FLAG_MESSAGE_RFC822=   0x04, /* Used by cached copy for messages */
  UDM_TEXTLIST_FLAG_HTML=             0x08  /* HTML format with entities */
} udm_textlist_flag_t;


typedef struct
{
  udm_secno_t secno;
  udm_textlist_flag_t flags;
} UDM_TEXT_PARAM;


typedef struct
{
  UDM_CONST_STR text;
  UDM_CONST_STR href;
  UDM_CONST_STR section_name;
} UDM_CONST_TEXTITEM;


typedef struct
{
  char  *str;
  char  *href;
  char  *section_name;
  UDM_TEXT_PARAM Param;
} UDM_TEXTITEM;


typedef struct
{
  size_t        nitems;
  size_t        mitems;
  UDM_TEXTITEM	*Item;
} UDM_TEXTLIST;

/*****************************************************/

/** StopList unit */
typedef struct udm_stopword_struct
{
  char *word;
} UDM_STOPWORD;

#define UDM_STOPLIST_LANGLEN 32
#define UDM_STOPLIST_CSETLEN 32
#define UDM_STOPLIST_FILELEN 128

typedef struct
{
  size_t nstopwords;
  UDM_STOPWORD	*StopWord;
  char  lang[UDM_STOPLIST_LANGLEN];
  char  cset[UDM_STOPLIST_CSETLEN];
  char  fname[UDM_STOPLIST_FILELEN];
} UDM_STOPLIST;

typedef struct
{
  size_t nitems;
  UDM_STOPLIST *Item;
} UDM_STOPLISTLIST;

/*****************************************************/

/** Words parameters */
typedef struct
{
  size_t min_word_len;
  size_t max_word_len;
} UDM_WORDPARAM;


/**************************/
typedef struct udm_coord_st
{
  udm_pos_t pos:24;   /* 3 */
  udm_secno_t secno;  /* 1 */
} UDM_COORD;

typedef struct udm_urlid_coord_st
{
  urlid_t url_id;     /* 4 */
  UDM_COORD coord;    /* 4 */
} UDM_URLID_COORD;

typedef struct
{
  UDM_URLID_COORD urlid_coord;
  udm_pos_t     seclen:24; /* 3 */  /*TODO34: get rid of this*/
  udm_wordnum_t num;       /* 1 */  /*TODO34: get rid of this*/
} UDM_URL_CRD;             /* 12 bytes total */


typedef struct
{
  size_t       acoords;
  size_t       ncoords;
  size_t       order;
  char	       *word;
  UDM_URL_CRD  *Coords;
} UDM_URLCRDLIST;


/***************************/
typedef struct udm_coord2_st
{
  udm_pos_t pos:24;
  udm_wordnum_t order;
} UDM_COORD2;


typedef struct udm_searchsection_st
{
  UDM_COORD2 *Coord;                 /* 4/8 bytes */
  const unsigned char *PackedCoord;  /* 4/8 bytes */
  urlid_t url_id;                    /* 4   bytes */
  udm_pos_t ncoords;                 /* 4   bytes */
  udm_pos_t seclen;                  /* 4   bytes */
  udm_pos_t minpos;                  /* 4   bytes */
  udm_pos_t maxpos;                  /* 4   bytes */
  udm_secno_t secno;                 /* 1   byte  */
  udm_wordnum_t wordnum;             /* 1   byte  */
  udm_wordnum_t order;               /* 1   byte  */
} UDM_SEARCHSECTION;                 /* 32 bytes (i386), 40 bytes (64bit) */


typedef struct udm_searchsectionlist_st
{
  size_t mcoords;
  size_t ncoords;
  UDM_COORD2 *Coord;
  size_t msections;
  size_t nsections;
  UDM_SEARCHSECTION *Section;
} UDM_SEARCHSECTIONLIST;


typedef struct udm_searchsectionlistlist_st
{
  size_t nitems;
  size_t mitems;
  UDM_SEARCHSECTIONLIST *Item;
} UDM_SEARCHSECTIONLISTLIST;



/** Main search structure */
typedef struct {
  urlid_t url_id;
  uint4   score;
} UDM_URL_SCORE;

typedef struct {
  size_t nitems;
  UDM_URL_SCORE	*Item;
} UDM_URLSCORELIST;


/* UserScore and UserSiteScore structure */

typedef struct udm_url_int4_st
{
  urlid_t url_id;
  int4  param;
} UDM_URL_INT4;

typedef struct udm_url_int4_list_st
{
  size_t nitems;
  UDM_URL_INT4 *Item;
} UDM_URL_INT4_LIST;



/* Structure to handle limits */
typedef struct udm_urlid_list_st
{
  char empty;
  char exclude;
  urlid_t *urls;
  size_t nurls;
} UDM_URLID_LIST;


typedef struct
{
  urlid_t   url_id;
  uint4     score;
  uint4     per_site;
  urlid_t   site_id;
  time_t    last_mod_time;
  double    pop_rank;
  char      *url;
  char      *section;
} UDM_URLDATA;

#define UDM_COORD2DBNUM(score) (255 - (int) ((score) & 0xFF))

typedef struct
{
  size_t       nitems;
  UDM_URLDATA  *Item;
} UDM_URLDATALIST;


/** Word list unit */
typedef struct
{
  char		*word;
  UDM_COORD     coord;
  unsigned char hash;
  unsigned char seclen_marker;
} UDM_WORD;

typedef struct
{
  size_t    wordpos[256]; /**< Word positions in sections               */
  size_t    mwords;       /**< Number of memory allocated for words     */
  size_t    nwords;       /**< Real number of words in list             */
  UDM_WORD *Word;         /**< Word list  itself                        */
} UDM_WORDLIST;


typedef struct
{
  size_t nitems;
  UDM_WORDLIST Item[256];
} UDM_WORDLISTLIST;


typedef struct
{
  const char *str;      /* 4 */
  udmcrc32_t crc;       /* 4 */
  UDM_COORD coord;      /* 4 */
  unsigned char length; /* 1 */
} UDM_CONSTWORD;


typedef struct
{
  size_t nitems;
  size_t mitems;
  UDM_CONSTWORD *Item;
  udm_pos_t wordpos[256];
} UDM_CONSTWORDLIST;

/***************************************************************/

/* Server/Realm follow types */
typedef enum
{
  UDM_WEBSPACE_PAGE= 0,
  UDM_WEBSPACE_PATH= 1,
  UDM_WEBSPACE_SITE= 2,
  UDM_WEBSPACE_WORLD= 3,
  UDM_WEBSPACE_URLLIST= 4,
  UDM_WEBSPACE_UNKNOWN= 127
} udm_webspace_t;
#define UDM_WEBSPACE_DEFAULT  UDM_WEBSPACE_PATH

typedef struct
{ 
  udm_bool_t index;   /**< Whether to index words         */
  udm_bool_t follow;  /**< Whether follow links           */
  udm_bool_t archive; /**< Whether to store cached copies */
} UDM_ROBOTSPARAM;


typedef struct
{
  int max_net_errors;
  int net_error_delay_time;
  int read_timeout;
  int doc_timeout;
  int period;                   /**< Reindex period           */
  int maxhops;                  /**< Max way in mouse clicks  */
  int doc_per_site;
  int crawl_delay;
  int dns_cache_timeout;
  int link_sources_to_follow;
  udm_link_destination_t collect_links_destination;
  udm_link_format_t      collect_links_format;
  udm_webspace_t webspace;      /**< World, Site, Path, Page  */
  udm_use_robots_t use_robots;  /**< Whether to use robots.txt and meta tags */
  UDM_ROBOTSPARAM robots;
  udm_bool_t use_clones;        /**< Whether to detect clones */
  udm_bool_t ajax_links;    /**< Whether to detect links with '#!' */
} UDM_SPIDERPARAM;

/*****************************************************************/

typedef enum
{
  UDM_MATCH_FLAG_NONE= 0,
  UDM_MATCH_FLAG_SKIP_OPTIMIZATION= 1,
  UDM_MATCH_FLAG_CASE_INSENSITIVE= 2,
  UDM_MATCH_FLAG_NEGATIVE= 4
} udm_matchflag_t;


typedef struct
{
  int one;
  int many;
  int eol;
} UDM_WILD_PARAM;


typedef struct
{
  udm_match_mode_t match_mode;
  udm_matchflag_t flags;      /* optimization, case sensitivity, negative */ 
} UDM_MATCH_PARAM;

typedef struct
{
  UDM_MATCH_PARAM Param;
#ifdef UDM_MATCH_PRIV
  UDM_STR Pattern;
#else
  UDM_STR m_hidden_Pattern;
#endif
  void *specific;
} UDM_MATCH;


typedef struct
{
  int beg;
  int end;
} UDM_MATCH_PART;


typedef struct
{
  UDM_MATCH Match;
  int quality;
} UDM_EXCERPT_FRAGMENT;


typedef struct
{
  UDM_MATCH Match;
  UDM_STR Replace;
} UDM_REPLACE;


typedef struct
{
  size_t nitems;
  size_t mitems;
  UDM_REPLACE *Item;
} UDM_REPLACELIST;


typedef struct
{
  UDM_REPLACE Replace;
  UDM_STR SectionName;
  UDM_STR Source;
} UDM_USERSECTION;


typedef struct
{
  size_t nitems;
  size_t mitems;
  UDM_USERSECTION *Item;
} UDM_USERSECTIONLIST;


typedef struct
{
  UDM_MATCH Match;
  udm_method_t method;
} UDM_FILTER;


typedef struct
{
  size_t mitems;
  size_t nitems;
  UDM_FILTER *Item;
} UDM_FILTERLIST;


typedef struct
{
  UDM_FILTER Filter;
  char *section;
} UDM_SECTIONFILTER;


typedef struct
{
  size_t mitems;
  size_t nitems;
  UDM_SECTIONFILTER *Item;
} UDM_SECTIONFILTERLIST;


/*****************************************************************/
/** Parsed URL string */
typedef struct udm_url
{
  char *schema;
  char *specific;
  char *hostinfo;
  char *auth;
  char *hostname;
  char *path;
  char *filename;
  char *anchor;
  int  port;
  int  default_port;
} UDM_URL;


/** List of URLs */
typedef struct udm_url_list
{
  size_t nitems;
  size_t mitems;
  UDM_URL *Item;
} UDM_URLLIST;


/*****************************************************************/

/** Structure to store server parameters */
typedef struct
{
  UDM_FILTER   Filter;
  urlid_t      site_id;        /**< server.rec_id            */
  char         command;        /**< 'S' - server,realm, 'F' - disallow,allow */
  int          ordre;          /**< order in list to find    */
  urlid_t      parent;         /**< parent rec_id for grouping by site */
  float        weight;         /**< server weight for popularity rank calculation */
  UDM_VARLIST  Vars;           /**< Default lang, charset,etc*/
  UDM_URLLIST  ProxyList;      /**< List of proxies */
  uint4        MaxHops;
  udm_webspace_t webspace;     /* Page, Path, Site, World, etc*/
  /*udm_method_t method;*/         /* Allow, Disallow, etc */
  udm_bool_t   enabled;
} UDM_SERVER;


typedef struct
{
  size_t      nservers;
  size_t      mservers;
  int         have_subnets;
  UDM_SERVER  *Server;
} UDM_SERVERLIST;


/*******************************************************/

typedef struct
{
  size_t  max_doc_per_site;
  urlid_t referrer;
  uint4	hops;
  urlid_t server_id;
  urlid_t rec_id;
  udm_method_t method;
  udm_bool_t stored;
  udm_link_source_t link_source;
  udm_link_destination_t method_reason;
} UDM_HREFPARAM;

/* All links are stored in the cache of this structure */
/* before actual INSERT into database                  */

typedef struct
{
  char *url;
  UDM_VARLIST HrefVars;
  UDM_HREFPARAM Param;
} UDM_HREF;


typedef enum
{
  UDM_HREFLIST_FLAG_NONUNIQ= 1
} udm_hreflist_flag_t;


typedef struct
{
  size_t    mhrefs;
  size_t    nhrefs;
  size_t    shrefs;
  UDM_HREF  *Href;
  int       flags;
} UDM_HREFLIST;

/*******************************************************/

/* IP with parameters */
typedef struct udm_sin_addr_st
{
  struct in_addr addr;
  time_t first_used;
  time_t last_used;
  size_t hits;
} UDM_INADDR;


typedef struct
{
  size_t nitems;
  size_t mitems;
  UDM_INADDR *Item;
} UDM_INADDRLIST;


/** Resolve stuff */
typedef struct udm_host_addr_struct
{
  char            *hostname;
  struct in_addr  addr;
  int             net_errors;
  time_t          last_used;
  time_t          expires;
} UDM_HOST_ADDR;


typedef struct
{
  size_t          nhost_addr;
  size_t          mhost_addr;
  UDM_HOST_ADDR  *host_addr;
} UDM_HOSTLIST;


/** Used in FTP sessions */
typedef struct udm_conn_struct
{
  int     status;
  int     connected;
  int     err;
  time_t  host_last_used;
  int     conn_fd;
#ifdef WIN32
  unsigned short port;
#else
  int     port;
#endif
  int     timeout;
  char    *hostname;
  char    *user;
  char    *pass;
  struct  sockaddr_in sin;
  int     buf_len;
  size_t  buf_len_total;
  int     len;
  char    *buf;
  int     net_errors;
  struct  udm_conn_struct *connp;
} UDM_CONN;

/***************************************************/

typedef struct
{
  char   *buf;           /**< Buffer to download document to    */
  char   *content;       /**< Pointer to content, after headers */
  size_t  size;          /**< Number of bytes loaded            */
  size_t  alloced_size;   /**< Maximum bytes to load into buf    */
} UDM_HTTPBUF;


typedef struct
{
  int stored;           /**< If it is already stored, forAddHref()   */
  udm_method_t method;  /**< How to download document: GET, HEAD etc */

  UDM_HTTPBUF     Buf;                  /**< Buffer       */

  UDM_HREFLIST    Hrefs;                /**< Link list    */

  UDM_VARLIST     RequestHeaders;       /**< Extra headers*/
  UDM_VARLIST     Sections;             /**< User sections*/

  UDM_TEXTLIST    TextList;             /**< Text list    */
  UDM_URL         CurURL;               /**< Parsed URL   */
  UDM_CHARSET     *lcs;                 /**< LocalCharser */
  UDM_SPIDERPARAM Spider;               /**< Spider prms  */
  UDM_CONN        connp;                /**< For FTP      */
  UDM_CONN        connp2;               /**< For FTP      */
} UDM_DOCUMENT;

/********************************************************/

/** External Parsers */
typedef struct udm_parser_struct
{
  char *from_mime;
  char *to_mime;
  char *cmd;
  char *src;
} UDM_PARSER;

typedef struct
{
  size_t      nparsers;
  UDM_PARSER  *Parser;
} UDM_PARSERLIST;


/******* Ispell BEGIN ********/

#define UDM_SPELL_NOPREFIX 1

typedef struct udm_spell_st
{
  char *word;
  char *flags;
} UDM_SPELL;

#define UDM_SPELL_LANGLEN 32
#define UDM_SPELL_CSETLEN 32
#define UDM_SPELL_FILELEN 128
#define UDM_SPELL_FMT_TEXT 0
#define UDM_SPELL_FMT_HASH 1

typedef struct udm_dict_st
{
  char  lang[UDM_SPELL_LANGLEN];
  char  cset[UDM_SPELL_CSETLEN];
  char  fname[UDM_SPELL_FILELEN];
  int   fmt;
  int   fd;
  size_t wordlen;
  UDM_CHARSET *cs;
  char   *fbody;
  size_t nitems;
  size_t mitems;
  UDM_SPELL *Item;
} UDM_SPELLLIST;


typedef struct udm_spelllistlist_st
{
  size_t nitems;
  size_t mitems;
  UDM_SPELLLIST *Item;
} UDM_SPELLLISTLIST;


typedef struct udm_aff_st UDM_AFFIX; /* Defined in spell.c */


typedef struct udm_afflist_st
{
  size_t mitems;
  size_t nitems;
  char  lang[UDM_SPELL_LANGLEN];
  char  cset[UDM_SPELL_CSETLEN];
  char  fname[UDM_SPELL_FILELEN];
  UDM_CHARSET *cs;
  UDM_AFFIX *Item;
} UDM_AFFIXLIST;


typedef struct udm_afflistlist_st
{
  size_t mitems;
  size_t nitems;
  UDM_AFFIXLIST *Item;
} UDM_AFFIXLISTLIST;


/******* Ispell END **********/


typedef struct
{
  udm_method_t  method; /**< 'allow' or 'disallow' */
  char         *path;
} UDM_ROBOT_RULE;


typedef struct
{
  char           *hostinfo;
  size_t          nrules;
  UDM_ROBOT_RULE  *Rule;
} UDM_ROBOT;


typedef struct
{
  size_t     nrobots;
  UDM_ROBOT  *Robot;
} UDM_ROBOTS;


/********************************************************/

typedef struct
{
  size_t           order; /*TODO34: change to udm_wordnum_t */
  size_t           order_extra_width; /* For multi-word synonyms, see below */
  size_t           count;
  size_t           doccount;    /* Number of documents this word appears in */
  udm_wordorigin_t origin;      /* query, spell, synonym, etc */
  int              weight;      /* origin-dependent weight   */
  int              user_weight; /* User-supplied weight */
  udm_match_mode_t match_mode;  /* BEGIN,END,SUBSTR,NUM_LT,NUM_GT,FULL*/
  size_t           secno;       /* Which section to search in; TODO34: udm_secno_t */
  size_t           phrpos;      /* 0 means "not in phrase"    */
  size_t           phrlen;      /* phase length               */
  size_t           phrwidth;    /* How many additional parts in a multi-word */
} UDM_WIDEWORD_PARAM;


typedef struct
{
  UDM_STR Word;
  UDM_WIDEWORD_PARAM Param;
} UDM_WIDEWORD;


/*
  order_extra_width - use in case of many-to-one and many-to-many synonyms.
  It represents the number of query words this synonym covers.
  For example, if we have synonym:
  "aaaa bbbb" -> "cccc"
  then origin_extra_width for the words "cccc" will be 2, because
  it is a replacement for two words.
  0 means "not a multi-word replacement".
*/

typedef struct
{
  udm_match_mode_t  match_mode;       /* Search mode: wrd, sub, beg, end */
  udm_bool_t        strip_noaccents;  /* If accent insensitive comparison*/ /* TODO34: change to a flag mask? */
  size_t            nuniq;
  size_t            nwords;
  UDM_WIDEWORD	    *Word;
} UDM_WIDEWORDLIST;


/*****************************************************************/

typedef struct
{
  char *p;
  char *s;
  udm_wordorigin_t origin; /* SYNONYM or SYNONYM_FINAL */
} UDM_SYNONYM;


#define UDM_SYNONYM_LANGLEN 32
#define UDM_SYNONYM_CSETLEN 32
#define UDM_SYNONYM_FILELEN 128
#define UDM_SYNONYM_FMT_TEXT 0
#define UDM_SYNONYM_FMT_HASH 1

typedef struct
{
  size_t      nsynonyms;
  size_t      msynonyms;
  UDM_SYNONYM *Synonym;
  char        lang[UDM_SYNONYM_LANGLEN];
  char        cset[UDM_SYNONYM_CSETLEN];
  char        fname[UDM_SYNONYM_FILELEN];
  size_t      max_phrase_length; /* Many-to-many, many-to-one, one-to-many */
} UDM_SYNONYMLIST;


typedef struct
{
  size_t nitems;
  UDM_SYNONYMLIST *Item;
} UDM_SYNONYMLISTLIST;


typedef struct udm_chinaword_struct
{
  int *word;
  int  freq;
} UDM_CHINAWORD;


typedef struct
{
  size_t        nwords;
  size_t        mwords;
  size_t        total;
  UDM_CHINAWORD *ChiWord;
  size_t        *hash;
} UDM_CHINALIST;


/*************************************************************/


/*** Boolean search constants and types ****/
typedef enum
{
  UDM_BOOLCMD_LEFT=    0,
  UDM_BOOLCMD_RIGHT=   1,
  UDM_BOOLCMD_BOT=     2,
  UDM_BOOLCMD_OR=      3,
  UDM_BOOLCMD_AND=     4,
  UDM_BOOLCMD_NOT=     5,
  UDM_BOOLCMD_PHRASE=  6,
  UDM_BOOLCMD_WORD=    200,
  UDM_BOOLCMD_STOP=    201
} udm_boolcmd_t;


typedef struct
{
  size_t  ncstack;
  size_t  mcstack;
  udm_boolcmd_t *cstack;
  size_t  nastack;
  size_t  mastack;
  unsigned long	*astack;
} UDM_BOOLSTACK; /* TODO34: split into two lists */


typedef struct
{
  udm_boolcmd_t cmd;
  unsigned long arg;
} UDM_BOOLEXPR_ITEM;


typedef struct
{
  size_t nitems;
  size_t mitems;
  size_t ncmds;
  UDM_BOOLEXPR_ITEM *items;
} UDM_BOOLEXPR;


/*****************************/
typedef struct
{
  size_t first;
  size_t last;
  size_t total_found;
} UDM_QUERY_STATS;


typedef struct
{
  size_t            num_rows;
  size_t            cur_row;
  UDM_DOCUMENT      *Doc;
  UDM_VARLIST       Vars;
  UDM_WIDEWORDLIST  WWList;
} UDM_RESULT;


typedef struct
{
  UDM_QUERY_STATS       stats;
  UDM_RESULT            Res;
  UDM_URLDATALIST       URLData;
  UDM_BOOLEXPR          BoolExpr;
  UDM_STATLIST          StatList;
  UDM_SEARCHSECTIONLIST SectionList;
  size_t num_best_rows;
  char *where;
  char *from;
} UDM_QUERY;


struct udm_dbhandler_st;

typedef struct udm_db_st
{  
  const struct udm_dbhandler_st *dbhandler;
  void *specific;
} UDM_DB;


typedef struct
{
  size_t  nitems;
  UDM_DB  *Item;
} UDM_DBLIST;


#define UDM_LOG_FLAG_SKIP_PID 0x00000001

typedef struct
{
  int is_log_open; /* Flag indicating if openlog() has been called      */
  FILE *logFD;     /* File descriptor, when logging to stderr or file   */
  int facility;    /* Which facility to use, or negative number if none */
  int flags;
  int level;
} UDM_LOG;


/****** SQLMon **************/

typedef enum
{
  UDM_MSG_DATA= 0,     /* Column values                       */
  UDM_MSG_ERROR= 1,    /* Error text                          */
  UDM_MSG_COMMAND= 2,  /* The command being executed          */
  UDM_MSG_EOL= 3,      /* End-of-line (e.g. record has ended) */
  UDM_MSG_PROMPT= 4,   /* ">SQL"                              */
  UDM_MSG_INFO= 5      /* e.g. "Connection changed to #1"     */  
} udm_msg_t;


/* Input/Output handler, e.g. for "indexer --sqlmon" */
typedef struct udm_iohandler_st
{
  void *user_data;
  char *(*gets)(struct udm_iohandler_st *prm, char *str, size_t size);
  udm_rc_t (*prompt)(struct udm_iohandler_st *prm, udm_msg_t msgtype, const char *msg);
} UDM_IOHANDLER;


/** Forward declaration of UDM_AGENT */
struct udm_indexer_struct_st;

typedef struct
{
  void (*Lock)(struct udm_indexer_struct_st *,
               udm_mutexcmd_t command, udm_mutexno_t number,
               const char *fname, int lineno);
  udm_rc_t (*ThreadCreate)(void *thd, void *(*start_routine)(void *), void *arg);
  udm_rc_t (*ThreadJoin)(void *thd);
} UDM_THDHANDLER;

#define UDM_GETLOCK(A,mutex)           if((A)->Conf->THDHandler.Lock)(A)->Conf->THDHandler.Lock((A),UDM_LOCK,(mutex),__FILE__,__LINE__)
#define UDM_RELEASELOCK(A,mutex)       if((A)->Conf->THDHandler.Lock)(A)->Conf->THDHandler.Lock((A),UDM_UNLOCK,(mutex),__FILE__,__LINE__)
#define UDM_LOCK_CHECK_OWNER(A,mutex)  if((A)->Conf->THDHandler.Lock)(A)->Conf->THDHandler.Lock((A),UDM_CKLOCK,(mutex),__FILE__,__LINE__)


typedef struct
{
  size_t maxlen[256];
} UDM_SECTIONPARAM;


/** Config file */
typedef struct udm_config_struct
{
  char                errstr[UDM_ERRSTR_SIZE];
  UDM_CHARSET         *bcs;
  UDM_CHARSET         *lcs;
  UDM_UNIDATA         *unidata;

  int                 url_number;      /**< For indexer -nXXX          */

  UDM_SERVERLIST      Servers;         /**< List of servers and realms */
  UDM_SERVER          *Cfg_Srv;

  UDM_REPLACELIST     Aliases;         /**< Straight aliases           */
  UDM_REPLACELIST     ReverseAliases;  /**< Reverse aliases            */
  UDM_REPLACELIST     MimeTypes;       /**< For AddType commands       */
  UDM_REPLACELIST     Encodings;       /**< For AddType commands       */

  UDM_FILTERLIST      Filters;         /**< Allow, Disallow,etc        */
  UDM_SECTIONFILTERLIST SectionFilters;  /**< IndexIf, NoIndexIf         */
  UDM_USERSECTIONLIST SectionHdrMatch; /**< User sections after headers*/
  UDM_USERSECTIONLIST SectionGsrMatch; /**< User sections after quesser*/
  UDM_USERSECTIONLIST SectionMatch;    /**< User sections after parser */

  UDM_HREFLIST        Hrefs;           /**< Links cache                */
  UDM_RESULT          Targets;         /**< Targets cache              */

  UDM_SECTIONPARAM    SectionParam;    /**< section parameters         */
  UDM_VARLIST         Sections;        /**< document section to parse  */
  UDM_VARLIST         Vars;            /**< Config parameters          */
  UDM_VARLIST         Cookies;         /**< Cookie list                */
  UDM_VARLIST         XMLEnterHooks;
  UDM_VARLIST         XMLLeaveHooks;
  UDM_VARLIST         XMLDataHooks;

  UDM_LANGMAPLIST     LangMaps;        /**< For lang+charset quesser   */
  UDM_ROBOTS          Robots;          /**< robots.txt information     */
  UDM_SYNONYMLISTLIST Synonym;         /**< Synonims list              */
  UDM_STOPLISTLIST    StopWord;        /**< Stopwords list             */
  UDM_PARSERLIST      Parsers;         /**< External  parsers          */
  UDM_DBLIST          DBList;          /**< Databases                  */
  UDM_HOSTLIST        Hosts;           /**< Resolve cache              */
  UDM_INADDRLIST      InAddr;          /**< IP cache                   */
  UDM_SPELLLISTLIST   Spells;          /**< For ispell dictionaries    */
  UDM_AFFIXLISTLIST   Affixes;         /**< For ispell affixes         */
  UDM_WORDPARAM       WordParam;       /**< Word limits                */
  UDM_CHINALIST       Chi;             /**< Chinese words list         */
  UDM_CHINALIST       Thai;            /**< Thai words list            */

  UDM_LOG Log;

  int    CVS_ignore;                   /**< Skip CVS directgories, for tests */

  /* Various virtual functions */
  UDM_THDHANDLER THDHandler;
  void (*ThreadInfo)(struct udm_indexer_struct_st *, const char * state, const char * str);
  void (*RefInfo)(int code, const char *url, const char *ref);
  int  (*DumpDoc)(struct udm_indexer_struct_st *, UDM_DOCUMENT *Doc);
#ifdef MECAB
   mecab_t         *mecab;
#endif

} UDM_ENV;


#define UDM_AGENT_STATE_MUTEXES 5

typedef struct udm_agent_state_t
{
  const char *task;
  const char *param;
  const char *extra;
  time_t start_time;
} UDM_AGENT_STATE;


/** Indexer */
typedef struct udm_indexer_struct_st
{
  udm_threadid_t  handle;      /**< Handler for threaded version */
  time_t          start_time;  /**< Time of allocation, for stat */
  udm_uint8       nbytes;      /**< Number of bytes downloaded   */
  size_t          ndocs;       /**< Number of documents          */
  size_t          nsleepsecs;  /**> Number of sleep seconds      */
  int             flags;       /**< Callback function to request action */
  udm_rc_t        action;
  int             doccount;    /**< for UdmGetDocCount()         */
  UDM_ENV         *Conf;       /**< Configuration                */
  UDM_LANGMAP     *LangMap;    /**< LangMap for current document */
  UDM_RESULT      Indexed;     /**< Indexed cache                */
  size_t          Indexed_memused; /**< Memory used by Indexed cache */

  UDM_AGENT_STATE State;

#ifdef USE_TRACE
  FILE *TR;
#endif

} UDM_AGENT;


typedef struct
{
  UDM_AGENT Agent;
  UDM_QUERY *Query;
} UDM_CRAWLER;


typedef struct udm_cfg_st
{
  UDM_AGENT   *Indexer;
  UDM_SERVER  *Srv;
  int         flags;
  int         level;
  int         ordre;
  size_t      excerpt_fragments_count;
} UDM_CFG;


typedef struct
{
  UDM_DSTR Value;
  char *name;
  size_t lineno;
} UDM_TEMPLATE_ITEM;


typedef struct
{
  size_t nitems;
  size_t mitems;
  UDM_TEMPLATE_ITEM *Item;
  int rc;      /* The code returned from the template program, e.g. exit() */
} UDM_TEMPLATE;


#ifdef DMALLOC
#include <dmalloc.h>
#endif

#define UDM_DT_BACK    1
#define UDM_DT_ER      2
#define UDM_DT_RANGE   3
#define UDM_DT_UNKNOWN 4

typedef enum
{
  UDM_OPT_BOOL,
  UDM_OPT_INT,
  UDM_OPT_STR,
  UDM_OPT_TITLE
} udm_opttype_t;


typedef struct udm_cmdline_opt_st
{
  int id;
  const char *name;
  int type;
  void *value;
  const char *comment;
} UDM_CMDLINE_OPT;


typedef enum
{
  UDM_IND_AMBIGUOUS=     0,
  UDM_IND_UNKNOWN=       1,
  UDM_IND_INDEX=         300,
  UDM_IND_CRAWL=         301,
  UDM_IND_STAT=          'S',
  UDM_IND_CREATE=        303,
  UDM_IND_DROP=          304,
  UDM_IND_DELETE=        'C',
  UDM_IND_REFERERS=      'I',
  UDM_IND_SQLMON=        'Q',
  UDM_IND_CHECKCONF=     308,
  UDM_IND_CONVERT=       309,
  UDM_IND_MULTI2BLOB=    310,
  UDM_IND_EXPORT=        311,
  UDM_IND_WRDSTAT=       312,
  UDM_IND_REWRITEURL=    313,
  UDM_IND_HASHSPELL=     314,
  UDM_IND_DUMPSPELL=     315,
  UDM_IND_REWRITELIMITS= 316,
  UDM_IND_DUMPCONF=      317,
  UDM_IND_DUMPDATA=      318,
  UDM_IND_RESTOREDATA=   319,
  UDM_IND_EXECSQL=       320,
  UDM_IND_SET=           321, /* indexer --set=a=b -> a=b    */
  UDM_IND_SET0=          322, /* indexer --fl=xxx  -> fl=xxx */
  UDM_IND_REWRITEPOP=    'R'
} udm_indcmd_t;

#endif
