/* Copyright (C) 2000-2015 Lavtech.com corp. All rights reserved.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
*/

#ifndef __INDEXCACHE_H__
#define __INDEXCACHE_H__

#define UDM_INDEXER_THREADS_MAX       64
#define INVERTED_INDEX_CACHE_PARTS     32
#define INVERTED_INDEX_CACHE_PART_SIZE ((4 * 1024 * 1024) / (INVERTED_INDEX_CACHE_PARTS))

struct udm_inverted_index_cache_st;

typedef struct
{
  udm_timer_t load;
  udm_timer_t sort;
  udm_timer_t pack;
  udm_timer_t send;
  udm_timer_t send_multi;
  udm_timer_t conv;
  udm_timer_t unpack_cached_copy;
  udm_timer_t parse;
  udm_timer_t prepare_words;
  udm_timer_t sort_wordlist;
  udm_timer_t pairs;
  unsigned long long bytes_loaded;
} UDM_INVERTED_INDEX_STATS;


typedef struct
{
  UDM_UNIDATA *unidata;
  UDM_ENV *Env;
  int cnvflags; /* e.g. for StripAccents */
  int pair_limit;
  int save_section_size;
  int aggregate_section_flags;
} UDM_INVERTED_INDEX_CACHE_PARAM;

typedef struct
{
  UDM_CONV cnv;
  urlid_t url_id;
  struct udm_inverted_index_cache_st *cache;
} UDM_CONSTWORD_HASH_DATA;


typedef struct
{
  char *ptr;                  /* 8 */ /* Word followed by encoded positions */
  urlid_t url_id;             /* 4 */
  uint4 length:24;            /* 3 */
  udm_secno_t secno;          /* 1 */
} UDM_INVERTED_INDEX_CACHE_ITEM;


typedef struct
{
  size_t nitems;                       /* 8 */
  size_t mitems;                       /* 8 */
  UDM_INVERTED_INDEX_CACHE_ITEM *Item; /* 8 */
} UDM_INVERTED_INDEX_CACHE_PART;


typedef struct
{
  char *str;                      /*  8 */
  udmhash32_t crc;                /*  4 */
  urlid_t last_url_id;            /*  4 */
  udm_pos_t last_url_id_count:23; /*    */
  int is_stopword:1;              /*    */
  unsigned int length:8;          /*    */
} UDM_INVERTED_INDEX_WORD;

typedef struct
{
  size_t nitems;
  size_t mitems;
  UDM_INVERTED_INDEX_WORD *Item;
} UDM_INVERTED_INDEX_WORD_LIST;


typedef struct
{
  UDM_MEMROOT root;
  UDM_HASH hash;
  UDM_INVERTED_INDEX_WORD_LIST list;
} UDM_INVERTED_INDEX_WORD_DICTIONARY;


typedef struct
{
  uint4 word_id;   /* 4 */
  UDM_COORD coord; /* 4 */
} UDM_INVERTED_INDEX_COORD;


typedef struct
{
  size_t nitems;
  size_t mitems;
  UDM_INVERTED_INDEX_COORD *Item;
} UDM_INVERTED_INDEX_COORD_LIST;


typedef struct udm_inverted_index_cache_st
{
  size_t nitems;
  size_t mitems;
  UDM_INVERTED_INDEX_CACHE_PART *Item;
  UDM_MEMROOT coord_root;
  UDM_INVERTED_INDEX_CACHE_PARAM param;
  UDM_INVERTED_INDEX_WORD_DICTIONARY Words;
  UDM_INVERTED_INDEX_STATS Stats;
} UDM_INVERTED_INDEX_CACHE;


void UdmInvertedIndexCachePartSort(UDM_INVERTED_INDEX_CACHE_PART *part);
void UdmInvertedIndexCachePartInit(UDM_INVERTED_INDEX_CACHE_PART *part);
void UdmInvertedIndexCachePartFree(UDM_INVERTED_INDEX_CACHE_PART *part);
udm_rc_t UdmInvertedIndexCacheAdd(UDM_AGENT *A,
                                  UDM_CONSTWORD_HASH_DATA *data,
                                  UDM_INVERTED_INDEX_CACHE *cache,
                                  const UDM_WORD *W, size_t ncoords);
udm_rc_t UdmInvertedIndexCacheAllocParts(UDM_AGENT *A,
                                         UDM_INVERTED_INDEX_CACHE *cache,
                                         size_t n);

void UdmInvertedIndexCacheInit(UDM_INVERTED_INDEX_CACHE *L, UDM_ENV *Env);
void UdmInvertedIndexCacheReset(UDM_INVERTED_INDEX_CACHE *L);
void UdmInvertedIndexCacheFree(UDM_INVERTED_INDEX_CACHE *L);
udm_rc_t UdmInvertedIndexCacheAddPart(UDM_AGENT *A, UDM_INVERTED_INDEX_CACHE *cache);

udm_rc_t UdmConstWordListToInvertedIndexCoordList(UDM_INVERTED_INDEX_CACHE *cache,
                                             UDM_INVERTED_INDEX_COORD_LIST *CL,
                                             UDM_CONSTWORDLIST *CWL);

udm_rc_t UdmInvertedIndexCoordListInit(UDM_INVERTED_INDEX_COORD_LIST *L, size_t mitems);
void UdmInvertedIndexCoordListFree(UDM_INVERTED_INDEX_COORD_LIST *L);
void UdmInvertedIndexCoordListSort(UDM_INVERTED_INDEX_COORD_LIST *L);

udm_rc_t UdmInvertedIndexCoordList2InvertedIndexCache(UDM_AGENT *A,
                                                     UDM_INVERTED_INDEX_COORD_LIST *CL,
                                                     UDM_CONSTWORDLIST *CWLWithPos,
                                                     UDM_CONSTWORD_HASH_DATA *data,
                                                     UDM_INVERTED_INDEX_CACHE *L);
size_t UdmInvertedIndexCacheEstimateUsedMemory(const UDM_INVERTED_INDEX_CACHE *);

#endif
