/* Copyright (C) 2000-2015 Lavtech.com corp. All rights reserved.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
*/

#ifndef _UDM_MATCH_H
#define _UDM_MATCH_H

void UdmMatchInit(UDM_MATCH *Match);
void UdmMatchFree(UDM_MATCH *Match);
void UdmMatchFreeAndInit(UDM_MATCH *Match);
void UdmMatchParamInit(UDM_MATCH_PARAM *Param);

void UdmMatchParamSetCaseInsensitive(UDM_MATCH_PARAM *M, udm_bool_t on);
void UdmMatchParamSetNegative(UDM_MATCH_PARAM *M, udm_bool_t on);
void UdmMatchParamSetOptimization(UDM_MATCH_PARAM *M, udm_bool_t on);

udm_rc_t UdmMatchSetPattern(UDM_MATCH *Match, const char *pattern);
udm_rc_t UdmMatchComp(UDM_MATCH *Match, char *errstr, size_t errstrsize);
udm_rc_t UdmMatchCompSpecificRegex(UDM_MATCH *Match, const char *pattern,
                                   char *errstr, size_t errstrsize);
int  UdmMatchExec(const UDM_MATCH *Match, const char *string, size_t stringlen,
                  const char *net_string, size_t nparts, UDM_MATCH_PART *Parts);
size_t UdmMatchApply(char *res, size_t ressize, const char *src,
                     const char *replacement, size_t replacementlen,
                     const UDM_MATCH *Match, size_t nparts,
                     UDM_MATCH_PART *Parts);

const char *UdmMatchModeStr(udm_match_mode_t m);
const char *UdmMatchCaseSensitivityStr(const UDM_MATCH *Match);
udm_bool_t UdmMatchIsCaseInsensitive(const UDM_MATCH *Match);
udm_bool_t UdmMatchIsNegative(const UDM_MATCH *Match);

void UdmReplaceListInit(UDM_REPLACELIST *L);
void UdmReplaceListFree(UDM_REPLACELIST *L);
udm_rc_t UdmReplaceListAdd(UDM_REPLACELIST *L,
                           const UDM_MATCH_PARAM *MatchParam,
                           const UDM_CONST_STR *pattern,
                           const UDM_CONST_STR *replace,
                           char *err, size_t errsize);
const UDM_REPLACE *UdmReplaceListFind(const UDM_REPLACELIST *L,
                                      const char *str,
                                      size_t nparts, UDM_MATCH_PART *Parts);

void UdmUserSectionListInit(UDM_USERSECTIONLIST *L);
void UdmUserSectionListFree(UDM_USERSECTIONLIST *L);
udm_rc_t UdmUserSectionListAdd(UDM_USERSECTIONLIST *L,
                               const UDM_MATCH_PARAM *M,
                               const UDM_CONST_STR *Section,
                               const UDM_CONST_STR *Source,
                               const UDM_CONST_STR *Pattern,
                               const UDM_CONST_STR *Replacement,
                               char *err, size_t errsize);
const UDM_USERSECTION *UdmUserSectionListFind(const UDM_USERSECTIONLIST *List,
                                              const char *string,
                                              size_t nparts,
                                              UDM_MATCH_PART *Parts);
void UdmFilterListInit(UDM_FILTERLIST *L);
void UdmFilterListFree(UDM_FILTERLIST *L);
udm_rc_t UdmFilterListAdd(UDM_FILTERLIST *L,
                          const UDM_MATCH_PARAM *M,
                          udm_method_t method,
                          const UDM_CONST_STR *Pattern,
                          char *err, size_t errsize);
const UDM_FILTER *UdmFilterListFind(const UDM_FILTERLIST *L, const char *str);

void UdmSectionFilterListInit(UDM_SECTIONFILTERLIST *L);
void UdmSectionFilterListFree(UDM_SECTIONFILTERLIST *L);
udm_rc_t UdmSectionFilterListAdd(UDM_SECTIONFILTERLIST *L,
                                 const UDM_MATCH_PARAM *M,
                                 udm_method_t method,
                                 const UDM_CONST_STR *Pattern,
                                 const char *section,
                                 char *err, size_t errsize);
const UDM_SECTIONFILTER *UdmSectionFilterListFind(const UDM_SECTIONFILTERLIST *L,
                                                  const char *str);

void UdmExcerptFragmentInit(UDM_EXCERPT_FRAGMENT *dst);
void UdmExcerptFragmentFree(UDM_EXCERPT_FRAGMENT *dst);

const char *UdmMatchPatternConstStr(const UDM_MATCH *M);
size_t UdmMatchPatternLength(const UDM_MATCH *M);

#endif
